/* =========================================================
 *   File ........... sysroot.cpp
 *   Purpose ........ sysroot class
 * ---------------------------------------------------------
 *   created ........ 15-05-2001
 *   last changed ... 09-06-2001
 *   by ............. MK
 * ========================================================= */

#include "globals.h"
#include "network.h"
#include "sysroot.h"
#include "primitive.h"
#include "avltree.h"
#include "world.h"
#include "worldinit.h"
#include "sound.h"


static transform_t rotate;
static transform *trans;
static float angle;
OID_t OID;


// ------- System Root --------

sysroot::sysroot()
{
	World = new world();
	//Network = new network();
	GameObj = new avltree;

	collisionobject = NULL;

	newcollision = -1;
	lastcollision = -1;
	lastgameobject = -1;
	contacttime = 0.0f;
}

sysroot::~sysroot()
{
	delete World;
	delete GameObj;
}


void 
sysroot::ThisControllerIs(controller *c)
{
	ThisController = c;
}

void 
sysroot::OtherControllerIs(controller *c)
{
	OtherController = c;
}

void 
sysroot::ThisControllerControls(player *p)
{
	ThisPlayer = p;
	ThisController->controls(ThisPlayer, World, this);
}

void 
sysroot::OtherControllerControls(player *p)
{
	OtherPlayer = p;
	OtherController->controls(OtherPlayer, World, this);
}


void 
sysroot::CreateWorld()
{
	material_t *m;
	baseobject *bo;

	m = material_create();
	material_setdiffuseambientcolor(m, 1.0f, 1.0f, 1.0f, 1.0f);
	material_texture(m,true,tex[SYSSHELL]);

	bo = createsphere(10.0f, 30, m);
	bo->update();

	trans = new transform();
	trans->push(&rotate);
	World->insertobject(bo, trans);

/*	
	gameobject *go = new gameobject(); // erzeugen eines gameobjects
	bo = createsphere(0.5f, 30, m);    // erzeugen einer geometrie
	bo->update();                       
	go->setgeometry(bo);               // dem gameobject wird eine gemoetry zugewiesen
	go->insertinto(World);             // die geometry eines gameobject wird in die weltr eingefgt
	GameObj->insert(go->getOID(), go); // das gameobject wird in die collision detection liste eingefgt
*/

    sysrootCs = createSysRoot();
	createApps(sysrootCs);
	createData(sysrootCs);
	appslist = sysrootCs->getlist();
}

void 
sysroot::Do(double duration)
{
	if (Network->is_connected()) Network->receive();
	ThisController->do_job(duration);
	OtherController->do_job(duration);

	if (contacttime <= 0.0f) if (ThisPlayer->hasmoved()) lastgameobject = -1;
	contacttime -= duration;

	newcollision = ThisController->checkcollision();
	if ((newcollision != -1) && (lastcollision != newcollision)) 
	{
printf("Collision with OID %d ", newcollision);
//printf("lc %i\n",lastcollision);
		gameobject *go = (gameobject *)(GameObj->search(newcollision));
		if (go != NULL) 
		{	
			printf ("/ OBJ_type %i\n");
			go->oncollision(World);
			if (go->OBJ_type & CYBERSTRUCTURE)
			{
				collisionobject = (cyberstructure *) go;
				play(COLLIS);
			}
			else play(ITEM_COLLECT); //XXX
		}
		lastgameobject = newcollision;
		contacttime = 0.5f;
	}
	lastcollision = newcollision;

//printf("%d\n", lastgameobject);

/*	if (IniFile.getintval("Rotation") != 0)
	{
		angle += (float)duration;
		trans_rotate(&rotate, angle, 0.0f, 1.0f, 0.0f);
	}
*/

	ToolMan->toolsecs+=duration;
	if ((ToolMan->toolsecs) >= 2.0) ToolMan->do_job(duration);

	toolList_t *tL = toolList;
	do
	{
		//oncollision for tool object
		/*if ((lastcollision != -1) & (tL->OID == OID)) 
		{
			printf("oncollision with OID: %i\n",OID);
			//tL->t->oncollision();
			ToolMan->hideTool(tL);
			ThisPlayer->addAbility(tL->t->getID());

		}
		*/

		//add tool to player abilities


		//rotate transformation
		if (tL->visible)
		{
			tL->t->angle+=(float)duration;
			trans_rotate(&(tL->t->rotate),tL->t->angle, tL->t->rotAx3dC->x,tL->t->rotAx3dC->y,tL->t->rotAx3dC->z);
		}
		tL=tL->next;
	}
	while (tL!=NULL);

	EventMan->do_job(duration);

	ThisController->view();

}
